import datetime
import unittest
import os
import tempfile
import io

import gb_io

DATA_FOLDER = os.path.realpath(os.path.join(__file__, os.path.pardir, "data"))


class TestDump(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        path = os.path.join(DATA_FOLDER, "AY048670.1.gb")
        cls.records = gb_io.load(path)
        with open(path) as src:
             cls.contents = src.read()

    def test_dump_single_record(self):
        buffer = io.BytesIO()
        gb_io.dump(self.records[0], buffer)
        lines_actual = buffer.getvalue().strip().decode().splitlines()
        lines_expected = self.contents.strip().splitlines()
        self.assertMultiLineEqual("\n".join(lines_actual[1:]), "\n".join(lines_expected[1:]))

    def test_dump_binary_file(self):
        buffer = io.BytesIO()
        gb_io.dump(self.records, buffer)
        lines_actual = buffer.getvalue().strip().decode().splitlines()
        lines_expected = self.contents.strip().splitlines()
        self.assertMultiLineEqual("\n".join(lines_actual[1:]), "\n".join(lines_expected[1:]))

    def test_dump_text_file(self):
        buffer = io.StringIO()
        gb_io.dump(self.records, buffer)
        lines_actual = buffer.getvalue().strip().splitlines()
        lines_expected = self.contents.strip().splitlines()
        self.assertMultiLineEqual("\n".join(lines_actual[1:]), "\n".join(lines_expected[1:]))

    def test_dump_path(self):
        with tempfile.NamedTemporaryFile(suffix=".gbk", mode="w+") as f:
            gb_io.dump(self.records, f.name)
            lines_actual = f.read().strip().splitlines()
        lines_expected = self.contents.strip().splitlines()
        self.assertMultiLineEqual("\n".join(lines_actual[1:]), "\n".join(lines_expected[1:]))

    def test_python_record(self):
        record = gb_io.Record(
            sequence=b"ATGC",
            name="Test sequence",
            source=gb_io.Source(name="Testus organismae"),
            date=datetime.date(2024, 4, 1),
            features=[
                gb_io.Feature("CDS", gb_io.Range(0, 3), [gb_io.Qualifier("translation", "M")]),
            ]
        )
        with tempfile.NamedTemporaryFile(suffix=".gbk", mode="w+") as f:
            gb_io.dump(record, f.name)
            lines_actual = f.read().strip().splitlines()
        self.assertEqual(
            lines_actual,
            [
                'LOCUS       Test sequence              4 bp            linear UNK 01-APR-2024',
                'SOURCE      Testus organismae',
                'FEATURES             Location/Qualifiers',
                '     CDS             1..3',
                '                     /translation="M"',
                'ORIGIN      ',
                '        1 ATGC',
                '//'
            ]
        )

class TestDumpError(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        path = os.path.join(DATA_FOLDER, "AY048670.1.gb")
        cls.records = gb_io.load(path)

    def test_dump_directory(self):
        dirname = os.path.dirname(__file__)
        self.assertRaises(OSError, gb_io.dump, self.records, dirname)

    def test_dump_bad_file(self):
        self.assertRaises(TypeError, gb_io.dump, self.records, None)

    def test_dump_none(self):
        buffer = io.BytesIO()
        dirname = os.path.dirname(__file__)
        self.assertRaises(TypeError, gb_io.dump, None, buffer)

    def test_dump_none_list(self):
        buffer = io.BytesIO()
        dirname = os.path.dirname(__file__)
        self.assertRaises(TypeError, gb_io.dump, [None], buffer)



import datetime
from typing import Optional, Union, BinaryIO, List, Iterator, Iterable

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal  # type: ignore

__author__: str
__version__: str

_STRAND = Literal["+", "-"]

class Record:
    name: Optional[str]
    length: Optional[int]
    molecule_type: Optional[str]
    division: str
    definition: Optional[str]
    accession: Optional[str]
    version: Optional[str]
    dblink: Optional[str]
    keywords: Optional[str]
    circular: bool
    date: Optional[datetime.date]
    sequence: bytearray
    features: List[Feature]
    references: List[Reference]
    def __init__(
        self,
        sequence: Union[bytes, bytearray, memoryview],
        *,
        name: Optional[str] = None,
        length: Optional[str] = None,
        molecule_type: Optional[str] = None,
        division: str = "UNK",
        definition: Optional[str] = None,
        accession: Optional[str] = None,
        version: Optional[str] = None,
        dblink: Optional[str] = None,
        keywords: Optional[str] = None,
        circular: bool = False,
        date: Optional[datetime.date] = None,
        source: Optional[Source] = None,
        contig: Optional[Location] = None,
        references: Optional[Iterable[Reference]] = None,
        features: Optional[Iterable[Feature]] = None,
    ): ...

class Source:
    name: str
    organism: Optional[str]
    def __init__(self, name: str, organism: Optional[str]): ...
    def __repr__(self) -> str: ...

class Feature:
    kind: str
    location: Location
    qualifiers: List[Qualifier]
    def __init__(
        self, kind: str, location: Location, qualifiers: Optional[List[Qualifier]]
    ): ...
    def __repr__(self) -> str: ...

class Qualifier:
    key: str
    value: Optional[str]
    def __init__(self, key: str, value: Optional[str] = None): ...
    def __repr__(self) -> str: ...

class Location:
    pass

class Range(Location):
    start: int
    end: int
    before: bool
    after: bool
    @property
    def strand(self) -> _STRAND: ...
    def __init__(
        self, start: int, end: int, before: bool = False, after: bool = False
    ): ...
    def __repr__(self) -> str: ...

class Between(Location):
    start: int
    end: int
    @property
    def strand(self) -> _STRAND: ...
    def __init__(self, start: int, end: int): ...
    def __repr__(self) -> str: ...

class Complement(Location):
    location: Location
    @property
    def start(self) -> int: ...
    @property
    def end(self) -> int: ...
    @property
    def strand(self) -> _STRAND: ...
    def __init__(self, location: Location): ...
    def __repr__(self) -> str: ...

class Join(Location):
    locations: List[Location]
    @property
    def start(self) -> int: ...
    @property
    def end(self) -> int: ...
    def __init__(self, locations: List[Location]): ...
    def __repr__(self) -> str: ...

class Order(Location):
    locations: List[Location]
    def __init__(self, locations: List[Location]): ...
    def __repr__(self) -> str: ...

class Bond(Location):
    locations: List[Location]
    def __init__(self, locations: List[Location]): ...
    def __repr__(self) -> str: ...

class OneOf(Location):
    locations: List[Location]
    def __init__(self, locations: List[Location]): ...
    def __repr__(self) -> str: ...

class External(Location):
    accession: str
    location: Optional[Location]
    def __init__(self, accession: str, location: Optional[Location] = None): ...
    def __repr__(self) -> str: ...

class Reference:
    description: str
    title: str
    authors: Optional[str]
    consortium: Optional[str]
    journal: Optional[str]
    pubmed: Optional[str]
    remark: Optional[str]

def load(fh: Union[str, BinaryIO]) -> List[Record]: ...
def iter(fh: Union[str, BinaryIO]) -> Iterator[Record]: ...
def dump(
    records: Union[Record, Iterable[Record]],
    fh: Union[str, BinaryIO],
    escape_locus: bool = False,
    truncate_locus: bool = False,
): ...



import unittest
import os

import gb_io

DATA_FOLDER = os.path.realpath(os.path.join(__file__, os.path.pardir, "data"))


class TestLoad(unittest.TestCase):

    def test_load_binary_file(self):
        path = os.path.join(DATA_FOLDER, "AY048670.1.gb")
        with open(path, "rb") as f:
            records = gb_io.load(f)

    def test_load_text_file(self):
        path = os.path.join(DATA_FOLDER, "AY048670.1.gb")
        with open(path, "rb") as f:
            records = gb_io.load(f)

    def test_load_path(self):
        path = os.path.join(DATA_FOLDER, "AY048670.1.gb")
        records = gb_io.load(path)


class TestLoadError(unittest.TestCase):

    def test_load_directory(self):
        dirname = os.path.dirname(__file__)
        self.assertRaises(OSError, gb_io.load, dirname)

    def test_load_file_not_found(self):
        path = "really/not/a/file/in/there"
        self.assertRaises(OSError, gb_io.load, path)

    def test_load_type_error(self):
        self.assertRaises(TypeError, gb_io.load, 1)
        self.assertRaises(TypeError, gb_io.load, [])

    def test_load_file_syntax_error(self):

        class Reader(object):
            def read(self, n):
                return b"LOCUS"

        r = Reader()
        self.assertRaises(ValueError, gb_io.load, r)

    def test_load_error_propagation(self):

        class MyError(ValueError):
            pass

        class Reader(object):
            def __init__(self):
                self.called = 0
            def read(self, n):
                if self.called == 0:
                    self.called += 1
                    return b"LOCUS"
                raise MyError("my error")

        r = Reader()
        self.assertRaises(MyError, gb_io.load, r)



import unittest
import os

import gb_io


class TestLocation(unittest.TestCase):

    def test_strand(self):
        location = gb_io.Range(1, 2)
        self.assertEqual(location.strand, "+")

        location = gb_io.Complement(location)
        self.assertEqual(location.strand, "-")